@@ -1,13 +1,14 @@
-use crate::config::{Alias, Config};
+use crate::config::{Alias, Config, PromptLevel};
 use crate::error::Result;
 use crate::list::Grouping;
 use crate::path;
-use crate::query::{EditQuery, FilterQuery, ListQuery, RangeQuery, ScriptQuery};
+use crate::query::{EditQuery, ListQuery, RangeQuery, ScriptQuery};
 use crate::script::ScriptName;
 use crate::script_type::ScriptType;
 use crate::tag::TagFilter;
 use crate::Either;
 use serde::Serialize;
+use std::path::PathBuf;
 use structopt::clap::AppSettings::{
     self, AllArgsOverrideSelf, AllowExternalSubcommands, AllowLeadingHyphen, DisableHelpFlags,
     DisableHelpSubcommand, DisableVersion, TrailingVarArg,
@@ -16,6 +17,8 @@ use structopt::StructOpt;
 
 mod completion;
 pub use completion::*;
+mod tags;
+pub use tags::*;
 
 const NO_FLAG_SETTINGS: &[AppSettings] = &[
     AllowLeadingHyphen,
@@ -26,72 +29,79 @@ const NO_FLAG_SETTINGS: &[AppSettings] = &[
     AllowExternalSubcommands,
 ];
 
-macro_rules! def_root {
-    ($sub:ident: $sub_type:ty) => {
-        #[derive(StructOpt, Debug, Serialize)]
-        #[structopt(settings = &[AllowLeadingHyphen, AllArgsOverrideSelf])]
-        pub struct Root {
-            #[structopt(skip = true)]
-            pub home_is_set: bool,
-
-            #[structopt(long, hidden = true)]
-            pub dump_args: bool,
-            #[structopt(long, global = true, help = "Do not record history")]
-            pub no_trace: bool,
-            #[structopt(short = "A", long, global = true, help = "Show scripts NOT within recent days", conflicts_with_all = &["all", "timeless"])]
-            pub archaeology: bool,
-            #[structopt(long, global = true)]
-            pub no_alias: bool, // NOTE: no-alias 的判斷其實存在於 structopt 之外，寫在這裡只是為了生成幫助訊息
-            #[structopt(short = "H", long, help = "Path to hyper script home")]
-            pub hs_home: Option<String>,
-            #[structopt(
-                short,
-                long,
-                global = true,
-                parse(try_from_str),
-                conflicts_with = "all",
-                number_of_values = 1,
-                help = "Filter by tags, e.g. `all,^mytag`"
-            )]
-            pub filter: Vec<TagFilter>,
-            #[structopt(
-                short,
-                long,
-                global = true,
-                conflicts_with = "recent",
-                help = "Shorthand for `-f=all,^removed --timeless`"
-            )]
-            all: bool,
-            #[structopt(long, global = true, help = "Show scripts within recent days.")]
-            pub recent: Option<u32>,
-            #[structopt(
-                long,
-                global = true,
-                help = "Show scripts of all time.",
-                conflicts_with = "recent"
-            )]
-            pub timeless: bool,
-
-            #[structopt(subcommand)]
-            pub $sub: Option<$sub_type>,
-        }
-    };
+#[derive(StructOpt, Debug, Serialize)]
+pub struct RootArgs {
+    #[structopt(long, hidden = true)]
+    pub dump_args: bool,
+    #[structopt(long, global = true, help = "Do not record history")]
+    pub no_trace: bool,
+    #[structopt(short = "A", long, global = true, help = "Show scripts NOT within recent days", conflicts_with_all = &["all", "timeless"])]
+    pub archaeology: bool,
+    #[structopt(long, global = true)]
+    pub no_alias: bool, // NOTE: no-alias 的判斷其實存在於 structopt 之外，寫在這裡只是為了生成幫助訊息
+    #[structopt(short = "H", long, help = "Path to hyper script home")]
+    pub hs_home: Option<String>,
+    #[structopt(
+        short,
+        long,
+        global = true,
+        parse(try_from_str),
+        conflicts_with = "all",
+        number_of_values = 1,
+        help = "Filter by tags, e.g. `all,^mytag`"
+    )]
+    pub filter: Vec<TagFilter>,
+    #[structopt(
+        short,
+        long,
+        global = true,
+        conflicts_with = "recent",
+        help = "Shorthand for `-f=all,^removed --timeless`"
+    )]
+    all: bool,
+    #[structopt(long, global = true, help = "Show scripts within recent days.")]
+    pub recent: Option<u32>,
+    #[structopt(
+        long,
+        global = true,
+        help = "Show scripts of all time.",
+        conflicts_with = "recent"
+    )]
+    pub timeless: bool,
+    #[structopt(long, possible_values(&["never", "always", "smart", "on-multi-fuzz"]), help = "Prompt level of fuzzy finder.")]
+    pub prompt_level: Option<PromptLevel>,
 }
 
-mod alias_mod {
-    use super::*;
-    #[derive(StructOpt, Debug, Serialize)]
-    pub enum Subs {
-        #[structopt(external_subcommand)]
-        Other(Vec<String>),
-    }
-    def_root! {
-        subcmd: Subs
-    }
+#[derive(StructOpt, Debug, Serialize)]
+#[structopt(settings = &[AllowLeadingHyphen, AllArgsOverrideSelf])]
+pub struct Root {
+    #[structopt(skip = false)]
+    #[serde(skip)]
+    home_is_set: bool,
+    #[structopt(flatten)]
+    pub root_args: RootArgs,
+    #[structopt(subcommand)]
+    pub subcmd: Option<Subs>,
+}
 
-    fn find_alias<'a>(root: &'a AliasRoot, conf: &'a Config) -> Option<(&'a Alias, &'a [String])> {
-        match &root.subcmd {
-            Some(alias_mod::Subs::Other(v)) => {
+#[derive(StructOpt, Debug, Serialize)]
+pub enum AliasSubs {
+    #[structopt(external_subcommand)]
+    Other(Vec<String>),
+}
+#[derive(StructOpt, Debug, Serialize)]
+#[structopt(settings = NO_FLAG_SETTINGS)]
+pub struct AliasRoot {
+    #[structopt(flatten)]
+    pub root_args: RootArgs,
+    #[structopt(subcommand)]
+    pub subcmd: Option<AliasSubs>,
+}
+impl AliasRoot {
+    fn find_alias<'a>(&'a self, conf: &'a Config) -> Option<(&'a Alias, &'a [String])> {
+        match &self.subcmd {
+            None => None,
+            Some(AliasSubs::Other(v)) => {
                 let first = v.first().unwrap().as_str();
                 if let Some(alias) = conf.alias.get(first) {
                     log::info!("別名 {} => {:?}", first, alias);
@@ -100,35 +110,27 @@ mod alias_mod {
                     None
                 }
             }
-            _ => None,
         }
     }
-    impl Root {
-        pub fn expand_alias<'a, T: 'a + AsRef<str>>(
-            &'a self,
-            args: &'a [T],
-            conf: &'a Config,
-        ) -> Option<impl Iterator<Item = &'a str>> {
-            if let Some((alias, remaining_args)) = find_alias(self, conf) {
-                let base_len = args.len() - remaining_args.len();
-                let base_args = args.iter().take(base_len).map(AsRef::as_ref);
-                let after_args = alias.after.iter().map(AsRef::as_ref);
-                let remaining_args = remaining_args.iter().map(AsRef::as_ref).skip(1);
-                let new_args = base_args.chain(after_args).chain(remaining_args);
+    pub fn expand_alias<'a, T: 'a + AsRef<str>>(
+        &'a self,
+        args: &'a [T],
+        conf: &'a Config,
+    ) -> Option<impl Iterator<Item = &'a str>> {
+        if let Some((alias, remaining_args)) = self.find_alias(conf) {
+            let base_len = args.len() - remaining_args.len();
+            let base_args = args.iter().take(base_len).map(AsRef::as_ref);
+            let after_args = alias.after.iter().map(AsRef::as_ref);
+            let remaining_args = remaining_args.iter().map(AsRef::as_ref).skip(1);
+            let new_args = base_args.chain(after_args).chain(remaining_args);
 
-                // log::trace!("新的參數為 {:?}", new_args);
-                Some(new_args)
-            } else {
-                None
-            }
+            // log::trace!("新的參數為 {:?}", new_args);
+            Some(new_args)
+        } else {
+            None
         }
     }
 }
-pub use alias_mod::Root as AliasRoot;
-
-def_root! {
-    subcmd: Subs
-}
 
 #[derive(StructOpt, Debug, Serialize)]
 #[structopt(settings = &[AllArgsOverrideSelf])]
@@ -189,8 +191,10 @@ pub enum Subs {
         dummy: bool,
         #[structopt(long, short, default_value = "1")]
         repeat: u64,
-        #[structopt(long, short)]
+        #[structopt(long, short, help = "Use arguments from last run")]
         previous_args: bool,
+        #[structopt(long, short, requires = "previous-args")]
+        dir: Option<PathBuf>,
         #[structopt(default_value = "-", parse(try_from_str))]
         script_query: ScriptQuery,
         #[structopt(help = "Command line args to pass to the script")]
@@ -243,15 +247,7 @@ pub enum Subs {
     #[structopt(
         about = "Manage script tags. If a tag filter is given, store it to config, otherwise show tag information."
     )]
-    Tags {
-        #[structopt(short, long, conflicts_with = "switch-activated")]
-        unset: Option<String>,
-        #[structopt(short, long, conflicts_with = "unset")]
-        switch_activated: Option<String>,
-
-        #[structopt(parse(try_from_str), conflicts_with_all = &["unset", "switch-activated"])]
-        tag_filter: Option<FilterQuery>,
-    },
+    Tags(Tags),
     #[structopt(about = "Manage script history")]
     History {
         #[structopt(subcommand)]
@@ -281,6 +277,8 @@ pub enum History {
         with_name: bool,
         #[structopt(short, long, default_value = "0")]
         offset: u32,
+        #[structopt(short, long)]
+        dir: Option<PathBuf>,
     },
     Neglect {
         #[structopt(parse(try_from_str), required = true, min_values = 1)]
@@ -341,31 +339,22 @@ fn print_help<S: AsRef<str>>(cmds: impl IntoIterator<Item = S>) -> Result {
 }
 
 fn handle_alias_args(args: Vec<String>) -> Result<Root> {
-    use structopt::clap::{Error as ClapError, ErrorKind::VersionDisplayed};
     if args.iter().any(|s| s == "--no-alias") {
         log::debug!("不使用別名！"); // NOTE: --no-alias 的判斷存在於 structopt 之外！
-        let mut root = Root::from_iter(args);
-        root.home_is_set = false;
+        let root = Root::from_iter(args);
         return Ok(root);
     }
     match AliasRoot::from_iter_safe(&args) {
         Ok(alias_root) => {
-            log::trace!("別名命令行物件 {:?}", alias_root);
-            set_home(&alias_root.hs_home)?;
-            if let Some(new_args) = alias_root.expand_alias(&args, Config::get()) {
-                // log::trace!("新的參數為 {:?}", new_args);
-                return Ok(Root::from_iter(new_args));
-            }
-            let root = Root::from_iter(args);
+            log::info!("別名命令行物件 {:?}", alias_root);
+            set_home(&alias_root.root_args.hs_home)?;
+            let mut root = match alias_root.expand_alias(&args, Config::get()) {
+                Some(new_args) => Root::from_iter(new_args),
+                None => Root::from_iter(&args),
+            };
+            root.home_is_set = true;
             Ok(root)
         }
-        Err(ClapError {
-            kind: VersionDisplayed,
-            ..
-        }) => {
-            // `from_iter_safe` 中已打印出版本，不再多做事，直接退出
-            std::process::exit(0);
-        }
         Err(e) => {
             log::warn!("解析別名參數出錯：{}", e); // NOTE: 不要讓這個錯誤傳上去，而是讓它掉入 Root::from_iter 中再來報錯
             Root::from_iter(args);
@@ -375,35 +364,34 @@ fn handle_alias_args(args: Vec<String>) -> Result<Root> {
 }
 
 impl Root {
-    /// 若帶了 --no-alias 選項，我們可以把設定腳本之家（以及載入設定檔）的時間再推遲
+    /// 若帶了 --no-alias 選項，或是補全模式，我們可以把設定腳本之家（以及載入設定檔）的時間再推遲
+    /// 在補全模式中意義重大，因為使用者可能會用 -H 指定別的腳本之家
     pub fn set_home_unless_set(&self) -> Result {
         if !self.home_is_set {
-            set_home(&self.hs_home)?;
+            set_home(&self.root_args.hs_home)?;
         }
         Ok(())
     }
     pub fn sanitize_flags(&mut self) {
-        if self.all {
-            self.timeless = true;
-            self.filter = vec!["all,^removed".parse().unwrap()];
+        if self.root_args.all {
+            self.root_args.timeless = true;
+            self.root_args.filter = vec!["all,^removed".parse().unwrap()];
         }
     }
     pub fn sanitize(&mut self) -> Result {
-        match &self.subcmd {
+        match &mut self.subcmd {
             Some(Subs::Other(args)) => {
-                let run = Subs::Run {
-                    dummy: false,
-                    previous_args: false,
-                    repeat: 1,
-                    script_query: args[0].parse()?,
-                    args: args[1..args.len()].to_vec(),
-                };
-                log::info!("執行模式 {:?}", run);
-                self.subcmd = Some(run);
+                let args =
+                    std::array::IntoIter::new(["hs", "run"]).chain(args.iter().map(|s| s.as_str()));
+                self.subcmd = Some(Subs::from_iter(args));
+                log::info!("執行模式 {:?}", self.subcmd);
             }
             Some(Subs::Help { args }) => {
                 print_help(args.iter())?;
             }
+            Some(Subs::Tags(tags)) => {
+                tags.sanitize();
+            }
             None => {
                 log::info!("無參數模式");
                 self.subcmd = Some(Subs::Edit {
@@ -423,7 +411,7 @@ impl Root {
 }
 
 pub fn handle_args(args: Vec<String>) -> Result<Either<Root, Completion>> {
-    if let Some(completion) = Completion::from_args(&args)? {
+    if let Some(completion) = Completion::from_args(&args) {
         return Ok(Either::Two(completion));
     }
     let mut root = handle_alias_args(args)?;
@@ -450,7 +438,7 @@ mod test {
     #[ignore = "structopt bug"]
     fn test_strange_set_alias() {
         let args = build_args("alias trash -f removed");
-        assert_eq!(args.filter, vec![]);
+        assert_eq!(args.root_args.filter, vec![]);
         match &args.subcmd {
             Some(Subs::Alias {
                 unset,
@@ -467,8 +455,8 @@ mod test {
     #[test]
     fn test_strange_alias() {
         let args = build_args("-f e e -t e something -T e");
-        assert_eq!(args.filter, vec!["e".parse().unwrap()]);
-        assert_eq!(args.all, false);
+        assert_eq!(args.root_args.filter, vec!["e".parse().unwrap()]);
+        assert_eq!(args.root_args.all, false);
         match &args.subcmd {
             Some(Subs::Edit {
                 edit_query,
@@ -486,8 +474,8 @@ mod test {
         }
 
         let args = build_args("la -l");
-        assert_eq!(args.filter, vec!["all,^removed".parse().unwrap()]);
-        assert_eq!(args.all, true);
+        assert_eq!(args.root_args.filter, vec!["all,^removed".parse().unwrap()]);
+        assert_eq!(args.root_args.all, true);
         match &args.subcmd {
             Some(Subs::LS(opt)) => {
                 assert_eq!(opt.long, true);
@@ -498,4 +486,46 @@ mod test {
             }
         }
     }
+    #[test]
+    fn test_default_run_tags() {
+        // TODO
+    }
+    #[test]
+    fn test_external_run_tags() {
+        let args = build_args("-f test --dummy -r 42 =script -a --");
+        assert_eq!(args.root_args.filter, vec!["test".parse().unwrap()]);
+        assert_eq!(args.root_args.all, false);
+        match args.subcmd {
+            Some(Subs::Run {
+                dummy: true,
+                previous_args: false,
+                repeat: 42,
+                dir: None,
+                script_query,
+                args,
+            }) => {
+                assert_eq!(script_query, "=script".parse().unwrap());
+                assert_eq!(args, vec!["-a", "--"]);
+            }
+            _ => {
+                panic!("{:?} should be run...", args);
+            }
+        }
+
+        let args = build_args("-f test --dump-args tags --name myname +mytag");
+        assert_eq!(args.root_args.filter, vec!["test".parse().unwrap()]);
+        assert_eq!(args.root_args.all, false);
+        assert!(args.root_args.dump_args);
+        match args.subcmd {
+            Some(Subs::Tags(Tags {
+                subcmd: Some(TagsSubs::Set { name, content }),
+            })) => {
+                assert_eq!(name, Some("myname".to_owned()));
+                assert_eq!(content, "+mytag".parse().unwrap());
+            }
+            _ => {
+                panic!("{:?} should be tags...", args);
+            }
+        }
+    }
 }
