@@ -1,15 +1,15 @@
-use hyper_scripter::args::{self, History, List, Root, Subs};
+use hyper_scripter::args::{self, History, List, Root, RootArgs, Subs, Tags, TagsSubs};
 use hyper_scripter::config::{Config, NamedTagFilter};
 use hyper_scripter::error::{Contextable, Error, RedundantOpt, Result};
 use hyper_scripter::extract_msg::{extract_env_from_content, extract_help_from_content};
 use hyper_scripter::list::{fmt_list, DisplayIdentStyle, DisplayStyle, ListOptions};
 use hyper_scripter::query::{self, RangeQuery, ScriptQuery};
-use hyper_scripter::script_repo::{RecentFilter, ScriptRepo};
+use hyper_scripter::script_repo::{RecentFilter, RepoEntry, ScriptRepo};
 use hyper_scripter::script_time::ScriptTime;
 use hyper_scripter::tag::TagFilter;
 use hyper_scripter::{
     path,
-    util::{self, main_util, main_util::EditTagArgs},
+    util::{self, main_util, main_util::EditTagArgs, print_iter},
 };
 use hyper_scripter_historian::Historian;
 
@@ -39,7 +39,7 @@ async fn main_err_handle() -> Result<Vec<Error>> {
     let args: Vec<_> = std::env::args().collect();
     let root = args::handle_args(args)?;
     let root = main_util::handle_completion(root)?;
-    if root.dump_args {
+    if root.root_args.dump_args {
         let dumped = serde_json::to_string(&root)?;
         print!("{}", dumped);
         return Ok(vec![]);
@@ -64,21 +64,33 @@ struct MainReturn {
 
 async fn main_inner(root: Root) -> Result<MainReturn> {
     root.set_home_unless_set()?;
+
+    let RootArgs {
+        no_trace,
+        archaeology,
+        filter,
+        recent,
+        timeless,
+        prompt_level,
+        ..
+    } = root.root_args;
+
+    Config::set_prompt_level(prompt_level);
     let conf = Config::get();
     let mut need_journal = main_util::need_write(root.subcmd.as_ref().unwrap());
     let (pool, init) = hyper_scripter::db::get_pool(&mut need_journal).await?;
 
-    let recent = if root.timeless {
+    let recent = if timeless {
         None
     } else {
-        root.recent.or(conf.recent).map(|recent| RecentFilter {
+        recent.or(conf.recent).map(|recent| RecentFilter {
             recent,
-            archaeology: root.archaeology,
+            archaeology: archaeology,
         })
     };
 
     let historian = Historian::new(path::get_home().to_owned()).await?;
-    let mut repo = ScriptRepo::new(pool, recent, historian.clone(), root.no_trace, need_journal)
+    let mut repo = ScriptRepo::new(pool, recent, historian.clone(), no_trace, need_journal)
         .await
         .context("讀取歷史記錄失敗")?;
 
@@ -89,14 +101,14 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
         main_util::load_templates()?;
     }
 
-    let explicit_filter = !root.filter.is_empty();
+    let explicit_filter = !filter.is_empty();
     let mut ret = MainReturn {
         conf: None,
         errs: vec![],
     };
     {
         let mut tag_group = conf.get_tag_filter_group(); // TODO: TagFilterGroup 可以多帶點 lifetime 減少複製
-        for filter in root.filter.into_iter() {
+        for filter in filter.into_iter() {
             tag_group.push(filter);
         }
         repo.filter_by_tag(&tag_group);
@@ -203,24 +215,18 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
         }
         Subs::Help { args } => {
             let script_query: ScriptQuery = args[0].parse()?;
-            let entry = query::do_script_query_strict(&script_query, &mut repo).await?;
+            let mut entry = query::do_script_query_strict(&script_query, &mut repo).await?;
             log::info!("檢視用法： {:?}", entry.name);
+            create_read_event(&mut entry).await?;
             let script_path = path::open_script(&entry.name, &entry.ty, Some(true))?;
             let content = util::read_file(&script_path)?;
 
             let helps = extract_help_from_content(&content);
-            for msg in helps {
-                println!("{}", msg);
-            }
+            print_iter(helps, "\n");
+            println!("");
+
             let envs = extract_env_from_content(&content);
-            let mut first = true;
-            for msg in envs {
-                if first {
-                    first = false;
-                    println!("");
-                }
-                println!("{}", msg);
-            }
+            print_iter(envs, "\n");
         }
         Subs::Run {
             script_query,
@@ -228,16 +234,18 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
             args,
             previous_args,
             repeat,
+            dir,
         } => {
             let mut entry = query::do_script_query_strict(&script_query, &mut repo).await?;
             main_util::run_n_times(
                 repeat,
                 dummy,
                 &mut entry,
-                &args,
+                args,
                 historian,
                 &mut ret.errs,
                 previous_args,
+                dir,
             )
             .await?;
         }
@@ -254,11 +262,12 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
             let script_path = path::open_script(&entry.name, &entry.ty, Some(true))?;
             let content = util::read_file(&script_path)?;
             print!("{}", content);
-            entry.update(|info| info.read()).await?;
+            create_read_event(&mut entry).await?;
         }
         Subs::EnvHelp { script_query } => {
-            let entry = query::do_script_query_strict(&script_query, &mut repo).await?;
+            let mut entry = query::do_script_query_strict(&script_query, &mut repo).await?;
             log::info!("打印 {:?} 的環境變數", entry.name);
+            create_read_event(&mut entry).await?;
             let script_path = path::open_script(&entry.name, &entry.ty, Some(true))?;
             let content = util::read_file(&script_path)?;
             let envs = extract_env_from_content(&content);
@@ -375,9 +384,9 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                 }
             }
         }
-        Subs::Tags {
-            unset: Some(name), ..
-        } => {
+        Subs::Tags(Tags {
+            subcmd: Some(TagsSubs::Unset { name }),
+        }) => {
             let conf = conf_mut!();
             let pos = conf
                 .tag_filters
@@ -389,10 +398,9 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                 })?;
             conf.tag_filters.remove(pos);
         }
-        Subs::Tags {
-            switch_activated: Some(name),
-            ..
-        } => {
+        Subs::Tags(Tags {
+            subcmd: Some(TagsSubs::Toggle { name }),
+        }) => {
             let conf = conf_mut!();
             let filter = conf
                 .tag_filters
@@ -404,40 +412,42 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                 })?;
             filter.inactivated = !filter.inactivated;
         }
-        Subs::Tags {
-            tag_filter: None, ..
-        } => {
-            print!("known tags:\n  ");
-            for t in repo.iter_known_tags() {
-                print!("{} ", t);
-            }
-            println!();
-            println!("tag filters:");
-            for filter in conf.tag_filters.iter() {
-                let content = &filter.content;
-                print!("  {} = [{}]", filter.name, content);
-                if content.mandatory {
+        Subs::Tags(Tags {
+            subcmd: Some(TagsSubs::LS { named, known }),
+        }) => {
+            if named {
+                print_iter(conf.tag_filters.iter().map(|f| &f.name), " ");
+            } else if known {
+                print_iter(repo.iter_known_tags(), " ");
+            } else {
+                print!("known tags:\n  ");
+                print_iter(repo.iter_known_tags(), " ");
+                println!("");
+                println!("tag filters:");
+                for filter in conf.tag_filters.iter() {
+                    let content = &filter.content;
+                    print!("  {} = [{}]", filter.name, content);
+                    if content.mandatory {
+                        print!(" (mandatory)")
+                    }
+                    if filter.inactivated {
+                        print!(" (inactivated)")
+                    }
+                    println!()
+                }
+                println!("main tag filter:");
+                print!("  [{}]", conf.main_tag_filter);
+                if conf.main_tag_filter.mandatory {
                     print!(" (mandatory)")
                 }
-                if filter.inactivated {
-                    print!(" (inactivated)")
-                }
-                println!()
+                println!();
             }
-            println!("main tag filter:");
-            print!("  [{}]", conf.main_tag_filter);
-            if conf.main_tag_filter.mandatory {
-                print!(" (mandatory)")
-            }
-            println!();
         }
-        Subs::Tags {
-            tag_filter: Some(filter),
-            ..
-        } => {
+        Subs::Tags(Tags {
+            subcmd: Some(TagsSubs::Set { content, name }),
+        }) => {
             let conf = conf_mut!();
-            let content = filter.content;
-            if let Some(name) = filter.name {
+            if let Some(name) = name {
                 log::debug!("處理篩選器 {:?}", name);
                 let tag_filters: &mut Vec<NamedTagFilter> = &mut conf.tag_filters;
                 if let Some(existing_filter) = tag_filters.iter_mut().find(|f| f.name == name) {
@@ -446,7 +456,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                 } else {
                     log::info!("新增篩選器 {} {}", name, content);
                     tag_filters.push(NamedTagFilter {
-                        content,
+                        content: content,
                         name,
                         inactivated: false,
                     });
@@ -520,10 +530,14 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                     limit,
                     with_name,
                     offset,
+                    dir,
                 },
         } => {
+            let dir = util::option_map_res(dir, |d| path::normalize_path(d))?;
             let entry = query::do_script_query_strict(&script, &mut repo).await?;
-            let args_list = historian.last_args_list(entry.id, limit, offset).await?;
+            let args_list = historian
+                .previous_args_list(entry.id, limit, offset, dir.as_deref())
+                .await?;
             for args in args_list {
                 log::debug!("嘗試打印參數 {}", args);
                 let args: Vec<String> = serde_json::from_str(&args)?;
@@ -533,14 +547,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
                         print!(" ");
                     }
                 }
-                let mut first = true;
-                for arg in args {
-                    if !first {
-                        print!(" ");
-                    }
-                    first = false;
-                    print!("{}", util::to_display_args(arg)?);
-                }
+                print_iter(args.into_iter().map(|s| util::to_display_args(s)), " ");
                 println!("");
             }
         }
@@ -548,3 +555,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
     }
     Ok(ret)
 }
+
+async fn create_read_event(entry: &mut RepoEntry<'_>) -> Result<i64> {
+    entry.update(|info| info.read()).await
+}
