@@ -46,7 +46,7 @@ impl Display for Error {
                     Regex => write!(f, "regular expression")?,
                     ScriptQuery => write!(f, "script query")?,
                     Tag => write!(f, "tag")?,
-                    FilterQuery => write!(f, "tag filter")?,
+                    PromptLevel => write!(f, "prompt level")?,
                     NonEmptyArray => {
                         write!(f, "non-empty array")?;
                         return Ok(());
