@@ -1,10 +1,11 @@
 use crate::error::{Contextable, Error, Result};
+use crate::script::IntoScriptName;
 use crate::script::{ScriptName, ANONYMOUS};
 use crate::script_type::ScriptType;
 use crate::state::State;
 use crate::util::{handle_fs_res, read_file};
 use std::fs::{canonicalize, create_dir, read_dir};
-use std::path::{Path, PathBuf};
+use std::path::{Component, Path, PathBuf};
 
 pub const HS_REDIRECT: &str = ".hs_redirect";
 pub const HS_PRE_RUN: &str = ".hs_prerun.sh";
@@ -53,17 +54,52 @@ fn get_sys_home() -> Result<PathBuf> {
     Ok(".test_hyper_scripter".into())
 }
 
-fn join_path<B: AsRef<Path>, P: AsRef<Path>>(base: B, path: P) -> Result<PathBuf> {
+fn join_here_abs<P: AsRef<Path>>(path: P) -> Result<PathBuf> {
+    join_path_abs(".", path)
+}
+fn join_path_abs<B: AsRef<Path>, P: AsRef<Path>>(base: B, path: P) -> Result<PathBuf> {
     let path = path.as_ref();
     if path.is_absolute() {
         return Ok(path.to_owned());
     }
-    let here = canonicalize(base)?;
-    Ok(here.join(path))
+    let base = base.as_ref();
+    // NOTE: 若 base 不存在，`canonicalize` 會造成問題
+    // 但前幾行已先處理掉絕對路徑，故出問題機會很小，而且這種問題本來就沒救了
+    let base_res = canonicalize(&base);
+    let base = handle_fs_res(&[base], base_res)?;
+    Ok(base.join(path))
+}
+
+pub fn normalize_path<P: AsRef<Path>>(path: P) -> Result<PathBuf> {
+    let path = join_here_abs(path)?;
+    let mut components = path.components().peekable();
+    let mut ret = if let Some(c @ Component::Prefix(..)) = components.peek().cloned() {
+        components.next();
+        PathBuf::from(c.as_os_str())
+    } else {
+        PathBuf::new()
+    };
+
+    for component in components {
+        match component {
+            Component::Prefix(..) => unreachable!(),
+            Component::RootDir => {
+                ret.push(component.as_os_str());
+            }
+            Component::CurDir => {}
+            Component::ParentDir => {
+                ret.pop();
+            }
+            Component::Normal(c) => {
+                ret.push(c);
+            }
+        }
+    }
+    Ok(ret)
 }
 
 fn compute_home_path<T: AsRef<Path>>(p: T) -> Result<PathBuf> {
-    let path = join_path(".", p)?;
+    let path = join_here_abs(p)?;
     log::debug!("計算路徑：{:?}", path);
     if !path.exists() {
         log::info!("路徑 {:?} 不存在，嘗試創建之", path);
@@ -100,7 +136,7 @@ pub fn get_home() -> &'static Path {
 }
 #[cfg(test)]
 pub fn get_home() -> &'static Path {
-    crate::set_once!(PATH, || { join_path(".", ".test_hyper_scripter").unwrap() });
+    crate::set_once!(PATH, || { join_here_abs(".test_hyper_scripter").unwrap() });
     PATH.get().as_ref()
 }
 
@@ -130,11 +166,9 @@ fn get_anonymous_ids() -> Result<Vec<u32>> {
 pub fn open_new_anonymous(ty: &ScriptType) -> Result<(ScriptName, PathBuf)> {
     let ids = get_anonymous_ids().context("無法取得匿名腳本編號")?;
     let id = ids.into_iter().max().unwrap_or_default() + 1;
-    Ok((ScriptName::Anonymous(id), open_anonymous(id, ty)?))
-}
-pub fn open_anonymous(id: u32, ty: &ScriptType) -> Result<PathBuf> {
-    let name = ScriptName::Anonymous(id);
-    open_script(&name, ty, None)
+    let name = id.into_script_name().unwrap();
+    let path = open_script(&name, ty, None)?; // NOTE: max + 1 的邏輯已足以確保不會產生衝突的腳本，不檢查了！
+    Ok((name, path))
 }
 
 /// 若 `check_exist` 有值，則會檢查存在性
@@ -176,7 +210,6 @@ pub fn get_template_path(ty: &ScriptType) -> Result<PathBuf> {
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::script::IntoScriptName;
     #[test]
     fn test_anonymous_ids() {
         let mut ids = get_anonymous_ids().unwrap();
@@ -189,12 +222,12 @@ mod test {
         assert_eq!(name, ScriptName::Anonymous(6));
         assert_eq!(
             p,
-            join_path("./.test_hyper_scripter/.anonymous", "6.sh").unwrap()
+            join_path_abs("./.test_hyper_scripter/.anonymous", "6.sh").unwrap()
         );
-        let p = open_anonymous(5, &"js".into()).unwrap();
+        let p = open_script(&5.into_script_name().unwrap(), &"js".into(), None).unwrap();
         assert_eq!(
             p,
-            join_path("./.test_hyper_scripter/.anonymous", "5.js").unwrap()
+            join_path_abs("./.test_hyper_scripter/.anonymous", "5.js").unwrap()
         );
     }
     #[test]
@@ -213,7 +246,7 @@ mod test {
         .unwrap();
         assert_eq!(
             p,
-            join_path("./.test_hyper_scripter/.anonymous", "1.sh").unwrap()
+            join_path_abs("./.test_hyper_scripter/.anonymous", "1.sh").unwrap()
         );
 
         match open_script(&not_exist, &"sh".into(), Some(true)).unwrap_err() {
