@@ -2,7 +2,7 @@ use super::{ListQuery, ScriptQuery, ScriptQueryInner};
 use crate::config::{Config, PromptLevel};
 use crate::error::{Error, Result};
 use crate::fuzzy;
-use crate::script::{IntoScriptName, ScriptInfo};
+use crate::script::ScriptInfo;
 use crate::script_repo::{RepoEntry, ScriptRepo};
 use crate::util::get_display_type;
 use fxhash::FxHashSet as HashSet;
@@ -67,7 +67,7 @@ pub async fn do_script_query<'b>(
     script_query: &ScriptQuery,
     script_repo: &'b mut ScriptRepo,
     force_all: bool,
-    force_prompt_level: Option<PromptLevel>,
+    forbid_prompt: bool,
 ) -> Result<Option<RepoEntry<'b>>> {
     log::debug!("開始尋找 `{:?}`", script_query);
     let all = script_query.bang || force_all;
@@ -83,26 +83,28 @@ pub async fn do_script_query<'b>(
         }
         ScriptQueryInner::Exact(name) => Ok(script_repo.get_mut(name, all)),
         ScriptQueryInner::Fuzz(name) => {
-            let level = force_prompt_level.unwrap_or(Config::get().prompt_level);
+            let level = if forbid_prompt {
+                PromptLevel::Never
+            } else {
+                Config::get_prompt_level()
+            };
             let fuzz_res = fuzzy::fuzz(name, script_repo.iter_mut(all), SEP).await?;
-            let need_prompt: bool;
+            let mut is_low = false;
+            let mut is_multi_fuzz = false;
             let entry = match fuzz_res {
-                Some(fuzzy::High(entry)) => {
-                    need_prompt = false;
-                    entry
-                }
+                Some(fuzzy::High(entry)) => entry,
                 Some(fuzzy::Low(entry)) => {
-                    need_prompt = true;
+                    is_low = true;
                     entry
                 }
                 #[cfg(feature = "benching")]
                 Some(fuzzy::Multi { ans, .. }) => {
-                    need_prompt = true;
+                    is_multi_fuzz = true;
                     ans
                 }
                 #[cfg(not(feature = "benching"))]
                 Some(fuzzy::Multi { ans, others }) => {
-                    need_prompt = true;
+                    is_multi_fuzz = true;
                     // NOTE: 從一堆分數相近者中選出最新的
                     // 但注意不要是「正解」的前綴，否則使用者可能永遠無法用模糊搜拿到名字比較短的候選者
                     // 例如 正解：ab 候選：ab1，且候選人較新
@@ -119,7 +121,15 @@ pub async fn do_script_query<'b>(
                 }
                 None => return Ok(None),
             };
-            if (need_prompt && !level.never()) || level.always() {
+            let need_prompt = {
+                match level {
+                    PromptLevel::Always => true,
+                    PromptLevel::Never => false,
+                    PromptLevel::Smart => is_low || is_multi_fuzz,
+                    PromptLevel::OnMultiFuzz => is_multi_fuzz,
+                }
+            };
+            if need_prompt {
                 prompt_fuzz_acceptable(&*entry)?;
             }
             Ok(Some(entry))
@@ -132,22 +142,19 @@ pub async fn do_script_query_strict<'b>(
 ) -> Result<RepoEntry<'b>> {
     // FIXME: 一旦 NLL 進化就修掉這段 unsafe
     let ptr = script_repo as *mut ScriptRepo;
-    if let Some(info) = do_script_query(script_query, unsafe { &mut *ptr }, false, None).await? {
+    if let Some(info) = do_script_query(script_query, unsafe { &mut *ptr }, false, false).await? {
         return Ok(info);
     }
 
     #[cfg(not(feature = "benching"))]
     if !script_query.bang {
-        let filtered =
-            do_script_query(script_query, script_repo, true, Some(PromptLevel::Never)).await?;
+        let filtered = do_script_query(script_query, script_repo, true, true).await?;
         if let Some(filtered) = filtered {
             return Err(Error::ScriptIsFiltered(filtered.name.key().to_string()));
         }
     }
 
-    Err(Error::ScriptNotFound(
-        script_query.clone().into_script_name()?.to_string(),
-    ))
+    Err(Error::ScriptNotFound(script_query.to_string()))
 }
 
 static CTRLC_HANDLE: Once = Once::new();
