@@ -37,7 +37,7 @@ impl FromStr for ScriptName {
     }
 }
 impl ScriptName {
-    pub fn valid(s: &str) -> Result<Option<u32>> {
+    pub fn valid(mut s: &str, fuzzing: bool) -> Result<Option<u32>> {
         log::debug!("檢查腳本名：{}", s);
         let reg = regex::Regex::new(r"^\.(\d+)$")?;
         let m = reg.captures(s);
@@ -48,6 +48,13 @@ impl ScriptName {
                 Err(e) => Err(Error::Format(ScriptNameCode, s.to_owned())).context(e),
             }
         } else {
+            if fuzzing {
+                if s == "." {
+                    return Ok(None); // NOTE: 讓匿名腳本可以直接用 `.` 來搜
+                } else if s.ends_with('/') {
+                    s = &s[..s.len() - 1]; // NOTE: 有了補全，很容易補出帶著`/`結尾的指令，放寬標準吧
+                }
+            }
             // FIXME: 好好想想什麼樣的腳本名可行，並補上單元測試
             for s in s.split('/') {
                 if illegal_name(s) {
@@ -154,9 +161,10 @@ pub struct ScriptInfo {
     pub read_time: ScriptTime,
     pub write_time: ScriptTime,
     pub neglect_time: Option<ScriptTime>,
-    /// (content, args)
-    pub exec_time: Option<ScriptTime<(String, String)>>,
-    pub exec_done_time: Option<ScriptTime<i32>>,
+    /// (content, args, dir)
+    pub exec_time: Option<ScriptTime<(String, String, Option<PathBuf>)>>,
+    /// (return code, main event id)
+    pub exec_done_time: Option<ScriptTime<(i32, i64)>>,
     #[deref]
     /// 用來區隔「時間資料」和「其它元資料」，並偵測其它元資料的修改
     timeless_info: TimelessScriptInfo,
@@ -222,15 +230,15 @@ impl ScriptInfo {
         self.read_time = now.clone();
         self.write_time = now;
     }
-    pub fn exec(&mut self, content: String, args: &[String]) {
+    pub fn exec(&mut self, content: String, args: &[String], dir: Option<PathBuf>) {
         log::trace!("{:?} 執行內容為 {}", self, content);
         let args_ser = serde_json::to_string(args).unwrap();
-        self.exec_time = Some(ScriptTime::now((content, args_ser)));
+        self.exec_time = Some(ScriptTime::now((content, args_ser, dir)));
         // NOTE: no readtime, otherwise it will be hard to tell what event was caused by what operation.
     }
-    pub fn exec_done(&mut self, code: i32) {
+    pub fn exec_done(&mut self, code: i32, main_event_id: i64) {
         log::trace!("{:?} 執行結果為 {}", self, code);
-        self.exec_done_time = Some(ScriptTime::now(code));
+        self.exec_done_time = Some(ScriptTime::now((code, main_event_id)));
     }
     pub fn neglect(&mut self) {
         self.neglect_time = Some(ScriptTime::now(()))
@@ -260,11 +268,16 @@ pub trait IntoScriptName {
     fn into_script_name(self) -> Result<ScriptName>;
 }
 
+impl IntoScriptName for u32 {
+    fn into_script_name(self) -> Result<ScriptName> {
+        Ok(ScriptName::Anonymous(self))
+    }
+}
 impl IntoScriptName for String {
     fn into_script_name(self) -> Result<ScriptName> {
         log::debug!("解析腳本名：{}", self);
-        if let Some(id) = ScriptName::valid(&self)? {
-            Ok(ScriptName::Anonymous(id))
+        if let Some(id) = ScriptName::valid(&self, false)? {
+            id.into_script_name()
         } else {
             Ok(ScriptName::Named(self))
         }
