@@ -13,9 +13,11 @@ use std::path::{Path, PathBuf};
 
 pub struct EditTagArgs {
     pub content: TagFilter,
+    /// 是否要把命名空間也做為標籤
     pub append_namespace: bool,
-
+    /// 命令行參數裡帶著 tag 選項，例如 hs edit --tag some-tag edit
     pub explicit_tag: bool,
+    /// 命令行參數裡帶著 filter 選項，例如 hs --filter some-tag edit
     pub explicit_filter: bool,
 }
 
@@ -61,7 +63,8 @@ pub async fn mv(
             }
             info.write();
         })
-        .await
+        .await?;
+    Ok(())
 }
 // XXX 到底幹嘛把新增和編輯的邏輯攪在一處呢…？
 pub async fn edit_or_create(
@@ -103,8 +106,8 @@ pub async fn edit_or_create(
             }};
         }
 
-        match query::do_script_query(&query, script_repo, false, None).await {
-            Err(Error::DontFuzz) => new_named!(),
+        match query::do_script_query(&query, script_repo, false, false).await {
+            Err(Error::DontFuzz) => new_named!(), // TODO: 手動測試文件？
             Ok(None) => new_named!(),
             Ok(Some(entry)) => {
                 if ty.is_some() {
@@ -125,6 +128,9 @@ pub async fn edit_or_create(
             Err(e) => return Err(e),
         }
     } else {
+        if tags.explicit_filter {
+            return Err(RedundantOpt::Filter.into());
+        }
         final_ty = ty.unwrap_or_default();
         log::debug!("打開新匿名腳本");
         path::open_new_anonymous(&final_ty).context("打開新匿名腳本失敗")?
@@ -155,33 +161,32 @@ pub async fn run_n_times(
     repeat: u64,
     dummy: bool,
     entry: &mut RepoEntry<'_>,
-    mut args: &[String],
+    mut args: Vec<String>,
     historian: Historian,
     res: &mut Vec<Error>,
-    previous_args: bool,
+    use_previous_args: bool,
+    dir: Option<PathBuf>,
 ) -> Result {
     log::info!("執行 {:?}", entry.name);
 
-    let previous_arg_vec: Vec<String>;
-    if previous_args {
-        if !args.is_empty() {
-            return Err(RedundantOpt::CommandArgs.into());
-        }
-        match historian.last_args(entry.id).await? {
+    if use_previous_args {
+        let dir = super::option_map_res(dir, |d| path::normalize_path(d))?;
+        match historian.previous_args(entry.id, dir.as_deref()).await? {
             None => return Err(Error::NoPreviousArgs),
             Some(arg_str) => {
                 log::debug!("撈到前一次呼叫的參數 {}", arg_str);
-                previous_arg_vec =
+                let mut previous_arg_vec: Vec<String> =
                     serde_json::from_str(&arg_str).context(format!("反序列失敗 {}", arg_str))?;
-                args = &previous_arg_vec;
-                // else 空字串當它是空陣列
+                previous_arg_vec.extend(args.into_iter());
+                args = previous_arg_vec;
             }
         }
     }
 
+    let here = path::normalize_path(".").ok();
     let script_path = path::open_script(&entry.name, &entry.ty, Some(true))?;
     let content = super::read_file(&script_path)?;
-    entry.update(|info| info.exec(content, args)).await?;
+    let run_id = entry.update(|info| info.exec(content, &args, here)).await?;
 
     if dummy {
         log::info!("--dummy 不用真的執行，提早退出");
@@ -194,7 +199,7 @@ pub async fn run_n_times(
             &*entry,
             &args,
             &entry.exec_time.as_ref().unwrap().data().unwrap().0,
-            entry.last_event_id(),
+            run_id,
         );
         let ret_code: i32;
         match run_res {
@@ -205,7 +210,9 @@ pub async fn run_n_times(
             Err(e) => return Err(e),
             Ok(_) => ret_code = 0,
         }
-        entry.update(|info| info.exec_done(ret_code)).await?;
+        entry
+            .update(|info| info.exec_done(ret_code, run_id))
+            .await?;
     }
     Ok(())
 }
@@ -261,10 +268,18 @@ pub fn need_write(arg: &Subs) -> bool {
     use Subs::*;
     match arg {
         Edit { .. } => true,
-        MV { .. } => true,
         CP { .. } => true,
         RM { .. } => true,
         LoadUtils { .. } => true,
+        MV {
+            ty,
+            tags,
+            new,
+            origin: _,
+        } => {
+            // TODO: 好好測試這個
+            ty.is_some() || tags.is_some() || new.is_some()
+        }
         _ => false,
     }
 }
@@ -305,6 +320,13 @@ pub fn handle_completion(root: Either<Root, Completion>) -> Result<Root> {
         Either::Two(completion) => match completion {
             Completion::LS { args } => {
                 let mut new_root = match Root::from_iter_safe(args) {
+                    Ok(Root {
+                        subcmd: Some(Subs::Tags(_)),
+                        ..
+                    }) => {
+                        // TODO: 在補全腳本中處理，而不要在這邊
+                        return Err(Error::Completion);
+                    }
                     Ok(t) => t,
                     Err(e) => {
                         log::warn!("補全時出錯 {}", e);
@@ -312,8 +334,7 @@ pub fn handle_completion(root: Either<Root, Completion>) -> Result<Root> {
                         return Err(Error::Completion);
                     }
                 };
-                new_root.home_is_set = false;
-                new_root.dump_args = false;
+                new_root.root_args.dump_args = false;
                 new_root.subcmd = Some(Subs::LS(List {
                     name: true,
                     plain: true,
@@ -332,7 +353,9 @@ pub fn handle_completion(root: Either<Root, Completion>) -> Result<Root> {
                             }
                         }
 
-                        let p = path::compute_home_path_optional(alias_root.hs_home.as_ref())?;
+                        let p = path::compute_home_path_optional(
+                            alias_root.root_args.hs_home.as_ref(),
+                        )?;
                         let conf = Config::load(&p)?;
                         if let Some(new_args) = alias_root.expand_alias(&args, &conf) {
                             print_iter(new_args);
