@@ -10,49 +10,49 @@ fn test_mv_same_name() {
     println!("移動腳本時，若前後腳本的路徑相同（腳本分類所致），應順利改動");
     let _g = setup();
 
-    run("e test | echo 1").unwrap();
-    run("e test2 | echo 2").unwrap();
+    run!("e test | echo 1").unwrap();
+    run!("e test2 | echo 2").unwrap();
 
-    run("mv test -T tmux").unwrap();
-    assert_eq!(run("test").unwrap(), "1");
+    run!("mv test -T tmux").unwrap();
+    assert_eq!(run!("test").unwrap(), "1");
 
-    run("mv test test2").expect_err("移動成撞名的腳本應報錯");
-    assert_eq!(run("test").unwrap(), "1");
-    run("mv test test2 -T rb").expect_err("移動成撞名的腳本，即使分類不同，也應報錯");
-    assert_eq!(run("test").unwrap(), "1");
+    run!("mv test test2").expect_err("移動成撞名的腳本應報錯");
+    assert_eq!(run!("test").unwrap(), "1");
+    run!("mv test test2 -T rb").expect_err("移動成撞名的腳本，即使分類不同，也應報錯");
+    assert_eq!(run!("test").unwrap(), "1");
 }
 #[test]
 fn test_cp_same_name() {
     println!("複製腳本時，若和既存的腳本撞名，應報錯");
     let _g = setup();
 
-    run("e test -T rb | puts 1").unwrap();
-    run("e test2 | echo 2").unwrap();
-    run("e test3 -T rb | puts 3").unwrap();
+    run!("e test -T rb | puts 1").unwrap();
+    run!("e test2 | echo 2").unwrap();
+    run!("e test3 -T rb | puts 3").unwrap();
 
-    run("cp test test2").expect_err("改成撞名的腳本，即使路徑不同，也應報錯");
-    assert_eq!(run("test2").unwrap(), "2");
+    run!("cp test test2").expect_err("改成撞名的腳本，即使路徑不同，也應報錯");
+    assert_eq!(run!("test2").unwrap(), "2");
 
-    run("cp test test3").expect_err("改成撞名的腳本應報錯");
-    assert_eq!(run("test3").unwrap(), "3");
+    run!("cp test test3").expect_err("改成撞名的腳本應報錯");
+    assert_eq!(run!("test3").unwrap(), "3");
 
-    assert_eq!(run("test").unwrap(), "1");
+    assert_eq!(run!("test").unwrap(), "1");
 }
 #[test]
 fn test_rm_non_exist() {
     println!("若欲刪除的腳本不存在，應直接消滅之。");
     let _g = setup();
 
-    run("e test | echo '刪我啊'").unwrap();
-    run("e test2 | echo '別刪我QQ'").unwrap();
-    assert_eq!(run("test").unwrap(), "刪我啊");
+    run!("e test | echo '刪我啊'").unwrap();
+    run!("e test2 | echo '別刪我QQ'").unwrap();
+    assert_eq!(run!("test").unwrap(), "刪我啊");
     let file = get_home().join("test.sh");
     remove_file(&file).unwrap();
     assert_ls_len(2, None, None);
-    assert_eq!(run("which test").unwrap(), file.to_string_lossy()); // TODO: 應該允許 which 嗎？
+    assert_eq!(run!("which test").unwrap(), file.to_string_lossy()); // TODO: 應該允許 which 嗎？
 
-    run("test").expect_err("刪掉的腳本還能執行！？");
-    run("rm *").expect("rm 應該要消滅掉不存在的腳本");
+    run!("test").expect_err("刪掉的腳本還能執行！？");
+    run!("rm *").expect("rm 應該要消滅掉不存在的腳本");
 
     assert_ls_len(1, Some("all"), None);
 }
@@ -61,18 +61,18 @@ fn test_hs_in_hs() {
     println!("若腳本甲裡呼叫了本程式去執行腳本乙，完成之後腳本甲的時間應較新");
     let _g = setup();
 
-    run(format!(
+    run!(
         "e outer | echo 我在第一層 && {} -H {} inner",
         get_exe_abs(),
         get_home().to_string_lossy(),
-    ))
+    )
     .unwrap();
-    run("e inner | echo '我在第幾層？'").unwrap();
+    run!("e inner | echo '我在第幾層？'").unwrap();
 
-    assert_eq!(run("-").unwrap(), "我在第幾層？");
-    assert_eq!(run("outer").unwrap(), "我在第一層\n我在第幾層？");
-    assert_eq!(run("-").unwrap(), "我在第一層\n我在第幾層？");
-    assert_eq!(run("^2").unwrap(), "我在第幾層？");
+    assert_eq!(run!("-").unwrap(), "我在第幾層？");
+    assert_eq!(run!("outer").unwrap(), "我在第一層\n我在第幾層？");
+    assert_eq!(run!("-").unwrap(), "我在第一層\n我在第幾層？");
+    assert_eq!(run!("^2").unwrap(), "我在第幾層？");
 }
 
 #[test]
@@ -80,14 +80,14 @@ fn test_remove_history_in_script() {
     println!("在腳本執行途中砍掉執行歷史，則該腳本的「執行完畢」事件應該一併消失");
     let _g = setup();
 
-    run("e test1 | echo 1").unwrap();
-    run("e test2 | echo 2 && $HS_EXE -H $HS_HOME history rm =${NAME}! 1").unwrap();
+    run!("e test1 | echo 1").unwrap();
+    run!("e test2 | echo 2 && $HS_EXE -H $HS_HOME history rm =${{NAME}}! 1").unwrap();
 
-    assert_eq!(run("-").unwrap(), "2");
-    assert_eq!(run("-").unwrap(), "2"); // 比較晚創造，所以刪了執行事件還是腳本2先
-    assert_eq!(run("test1").unwrap(), "1");
-    assert_eq!(run("test2").unwrap(), "2");
-    assert_eq!(run("-").unwrap(), "1");
+    assert_eq!(run!("-").unwrap(), "2");
+    assert_eq!(run!("-").unwrap(), "2"); // 比較晚創造，所以刪了執行事件還是腳本2先
+    assert_eq!(run!("test1").unwrap(), "1");
+    assert_eq!(run!("test2").unwrap(), "2");
+    assert_eq!(run!("-").unwrap(), "1");
 }
 
 #[test]
@@ -95,7 +95,7 @@ fn test_edit_existing_bang() {
     println!("用 BANG! 編輯已存在的腳本，不該出錯");
     let _g = setup();
 
-    run("e test -t hide | echo 躲貓貓").unwrap();
+    run!("e test -t hide | echo 躲貓貓").unwrap();
 
     // 當場變一個異步執行期出來。不要直接把測試函式寫成異步，否則 setup 中鎖的處理會出問題…
     let mut rt = tokio::runtime::Runtime::new().unwrap();
@@ -159,13 +159,13 @@ fn test_edit_without_change() {
     println!("沒有動到檔案就不要記錄寫入事件");
     let _g = setup();
 
-    run("e test1 | echo $NAME").unwrap();
-    run("e test2 | echo $NAME").unwrap();
-    assert_eq!("test2", run("-").unwrap());
-    run("e test1").unwrap(); // nothing changed! FIXME: 其實應該記錄一筆讀取事件…但是很難測…
-    assert_eq!("test2", run("-").unwrap());
-    run("e test1 | echo $NAME").unwrap();
-    assert_eq!("test1\ntest1", run("-").unwrap());
+    run!("e test1 | echo $NAME").unwrap();
+    run!("e test2 | echo $NAME").unwrap();
+    assert_eq!("test2", run!("-").unwrap());
+    run!("e test1").unwrap(); // nothing changed! FIXME: 其實應該記錄一筆讀取事件…但是很難測…
+    assert_eq!("test2", run!("-").unwrap());
+    run!("e test1 | echo $NAME").unwrap();
+    assert_eq!("test1\ntest1", run!("-").unwrap());
 }
 
 #[test]
@@ -221,14 +221,14 @@ fn test_history_rm_range() {
     t1.run("b").unwrap();
 
     fn show_history(script: &ScriptTest) -> Vec<String> {
-        let s = run(format!("history show {}", script.get_name())).unwrap();
+        let s = run!("history show {}", script.get_name()).unwrap();
         s.split('\n').map(|s| s.to_owned()).collect()
     }
 
     t1.can_find("-").expect("用最近期詢問找不到？");
     assert_eq!(show_history(&t1), vec!["b", "a", "sep"]);
 
-    run(format!("history rm {} 1..3", t1.get_name())).unwrap(); // rm b & a
+    run!("history rm {} 1..3", t1.get_name()).unwrap(); // rm b & a
     assert_eq!(show_history(&t1), vec!["sep"]);
 
     t2.can_find("-").expect("刪除整段事件未能影響近期詢問");
@@ -239,3 +239,20 @@ fn test_history_rm_range() {
         "另一個腳本的歷史爛掉了"
     );
 }
+
+#[test]
+fn test_fuzz_end_with_slash() {
+    println!("測試以`/`結尾的腳本名");
+    let _g = setup();
+
+    let t = ScriptTest::new("test/slash", None);
+    t.can_find("test/").unwrap();
+    t.can_find("t/").unwrap();
+    t.can_find("slash/").unwrap();
+
+    t.can_find("sla//").expect_err("兩個`/`結尾仍不可行");
+    t.can_find("/sla").unwrap_err();
+
+    run!("e illegal/").expect_err("不應創建以`/`結尾的腳本");
+    run!("mv - illegal/").expect_err("不應創建以`/`結尾的腳本");
+}
